# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import MessageV2, Worklist
from pandas import read_excel

logger = logging.getLogger("driverng.drivers.stp7000")


class Driver(MessageV2):
    SPEC = {"code": "stp7000", "label": "Extracteur STP7000"}

    def parse(self, data):
        """
        Convert the .xlsx file (data) into a DataFrame using the openpyxl engine.

        Example of DataFrame:

                0             1             2             3
        0  PlateID         12345           NaN           NaN
        1      NaN             1             2             3
        2        A  19E340041469  19E320083243  19E111106848
        3        B  19E290222772  19E240232355  19E320083236

        sheet_name=0 : specifies the first sheet is read.
        header=None : considers the columns doesn't have name.
        """
        return read_excel(data, sheet_name=0, header=None, engine="openpyxl")

    def get_geometry_format(self, value):
        """
        Retrieve geometry format depending the value of the axis reference.

        Returns:
            str: "n0" or "n1" if format is interger, "al" if format is alphabetical
        """
        try:
            int(value)
            return f"n{value}"
        except ValueError:
            return "al"

    def decode_message(self, message):
        """
        message = dataframe
        La worklist obtenue se présente de la façon suivante :

        worklist = {
            "context": "result",
                "origin": "plandeplaque",
                "samples": [
                    {
                        "geometry": {
                            "format_abscissa": "n1",
                            "format_ordinate": "al",
                            "height": 12,
                            "width": 8,
                        },
                        "sample_id": "12345",
                        "type": "patient",
                        "wells": [
                            {
                                "abscissa": "1",
                                "barcode": "19E340041469",
                                "ordinate": "A",
                                "well_type": 0,
                            },
                            {
                                "abscissa": "2",
                                "barcode": "19E320083243",
                                "ordinate": "A",
                                "well_type": 0,
                            },
                            {
                                "abscissa": "3",
                                "barcode": "19E111106848",
                                "ordinate": "A",
                                "well_type": 0,
                            },
                        ],
                    }
                ],
            }

        Type de position sur la plaque (well_type) :
        0 : Position (puit) patient
        1 : Position (puit) de contrôle
        """

        worklist = Worklist(context="result", origin="plandeplaque")
        if message.empty:
            logger.error("Le fichier Excel n'a pas pu être traité")
            return worklist
        else:
            # access a single value for a row/column pair by index
            sample_id = message.iloc[0, 1]
            # get the number of columns and rows of a dataframe
            dimensions = message.shape
            # convert the DataFrame into a dictionary like {index -> {column -> value}}
            excel_dict = message.to_dict("index")

            # Remove the first two lines retrieved not containing positions.
            excel_dict.pop(0)
            excel_dict.pop(1)

            sample = worklist.get_or_create(type="patient", sample_id=str(sample_id))

            if (str(message.iloc[2, 1])).lower() == "nan":
                logger.error("La plaque ne contient aucun échantillon")
                return worklist
            else:
                """
                For geometry:
                height = dimensions[0] (number of rows) - 2 (the first 2 rows)
                width = dimensions[1] (number of columns) - 1 (the first column)
                """
                sample.geometry.height = dimensions[0] - 2
                sample.geometry.width = dimensions[1] - 1
                sample.geometry.format_abscissa = self.get_geometry_format(
                    message.iloc[1, 1]
                )
                sample.geometry.format_ordinate = self.get_geometry_format(
                    message.iloc[2, 0]
                )

                for key in excel_dict:
                    rows = excel_dict.get(key)
                    row_name = rows.get(0)
                    well_type = 0
                    qc_suffix = 0
                    for key in rows:
                        if str(rows.get(key)) == "nan":
                            well_type = 1
                            qc_suffix += 1
                        if rows.get(key) == row_name:
                            continue
                        if qc_suffix > 2:
                            break
                        else:
                            sample.wells.get_or_create(
                                barcode=rows.get(key)
                                if isinstance(rows.get(key), str)
                                else "Control_{}".format(qc_suffix),
                                abscissa=str(key),
                                ordinate=row_name,
                                well_type=well_type,
                            )
                    if qc_suffix > 2:
                        break

                return worklist
